@@ -300,7 +300,7 @@ sub connect_database {
 			if (open $fh, '>', $readme) {
 				print $fh "This is a test directory for DBD::Pg and may be removed\n";
 				print $fh "You may want to ensure the postmaster has been stopped first.\n";
-				print $fh "Check the port in the postgresql.conf file\n";
+				print $fh "Check the data/postmaster.pid file\n";
 				close $fh or die qq{Could not close "$readme": $!\n};
 			}
 
@@ -312,46 +312,40 @@ sub connect_database {
 			unshift @userlist, $username if defined $username and $username ne getpwent;
 
 			my %doneuser;
-			for my $user (@userlist) {
-				next if $doneuser{$user}++;
-				$uid = (getpwnam $user)[2];
+			for (@userlist) {
+				$testuser = $_;
+				next if $doneuser{$testuser}++;
+				$uid = (getpwnam $testuser)[2];
 				next if !defined $uid;
 
 				next unless chown $uid, -1, $testdir;
 				next unless chown $uid, -1, $readme;
-				$su = $user;
+				$su = $testuser;
 				$founduser++;
 				$info = '';
 				my $olddir = getcwd;
 				eval {
 					chdir $testdir;
-					$info = qx{su -m $user -c "$initdb --locale=C -E UTF8 -D $testdir/data 2>&1"};
+					$info = qx{su -m $testuser -c "$initdb --locale=C -E UTF8 -D $testdir/data 2>&1"};
 				};
 				my $err = $@;
 				chdir $olddir;
-				if (!$err and $info =~ /owned by user "$user"/) {
-					$testuser = $user;
-					last;
-				}
+				last if !$err;
 			}
 			if (!$founduser) {
 				$@ = 'Unable to find a user to run initdb as';
-				last GETHANDLE;
-			}
-			if (!$testuser) {
-				$@ = "Failed to run initdb as user $su: $@";
-				last GETHANDLE;
+				last GETHANDLE; ## Fail - no user
 			}
 			if (! -e "$testdir/data") {
 				$@ = 'Could not create a test database via initdb';
-				last GETHANDLE;
+				last GETHANDLE; ## Fail - no datadir created
 			}
 			## At this point, both $su and $testuser are set
 		}
 
 		if ($info =~ /FATAL/) {
 			$@ = "initdb gave a FATAL error: $info";
-			last GETHANDLE;
+			last GETHANDLE; ## Fail - FATAL
 		}
 
 		if ($info =~ /but is not empty/) {
@@ -359,7 +353,7 @@ sub connect_database {
 		}
 		elsif ($info !~ /pg_ctl/) {
 			$@ = "initdb did not give a pg_ctl string: $info";
-			last GETHANDLE;
+			last GETHANDLE; ## Fail - bad output
 		}
 
 		## Which user do we connect as?
@@ -389,7 +383,7 @@ sub connect_database {
 			}
 			if ($testport >= $maxport) {
 				$@ = "No free ports found for testing: tried 5440 to $maxport\n";
-				last GETHANDLE;
+				last GETHANDLE; ## Fail - no free ports
 			}
 		}
 		$@ = '';
@@ -398,7 +392,7 @@ sub connect_database {
 		my $cfh;
 		if (! open $cfh, '>>', $conf) {
 			$@ = qq{Could not open "$conf": $!};
-			last GETHANDLE;
+			last GETHANDLE; ## Fail - no conf file
 		}
 		print $cfh "\n\n## DBD::Pg testing parameters\nport=$testport\nmax_connections=4\n";
 		print $cfh "listen_addresses='localhost'\n" if $^O =~ /Win32/;
@@ -437,7 +431,7 @@ sub connect_database {
 			$su and chdir $olddir;
 			if ($err or $info !~ /\w/) {
 				$@ = "Could not startup new database ($COM) ($err) ($info)";
-				last GETHANDLE;
+				last GETHANDLE; ## Fail - startup failed
 			}
 			sleep 1;
 		}
@@ -462,7 +456,7 @@ sub connect_database {
 					redo STARTUP;
 				}
 			}
-			last GETHANDLE;
+			last GETHANDLE; ## Made it!
 		}
 
 	} ## end of GETHANDLE
@@ -499,6 +493,10 @@ sub connect_database {
 	$ENV{DBI_DSN} = $testdsn;
 	$ENV{DBI_USER} = $testuser;
 
+	if ($arg->{quickreturn}) {
+		return $helpconnect, '', $dbh;
+	}
+
 	if ($arg->{nosetup}) {
 		return $helpconnect, '', $dbh unless schema_exists($dbh, $S);
 		$dbh->do("SET search_path TO $S");
@@ -530,6 +528,7 @@ CREATE TABLE dbd_pg_test (
   testarray  text[][],
   testarray2 int[],
   "CaseTest" boolean,
+  expo       numeric(6,2),
   bytetest   bytea
 )
 };
@@ -585,9 +584,13 @@ sub get_test_settings {
 	}
 	my ($testdsn, $testuser, $testdir, $error) = ('','','','?');
 	my ($helpconnect, $su, $uid, $initdb) = (0,'','','default');
+	my $inerror = 0;
 	if (-e $helpfile) {
 		open $fh, '<', $helpfile or die qq{Could not open "$helpfile": $!\n};
 		while (<$fh>) {
+			if ($inerror) {
+				$error .= "\n$_";
+			}
 			/DSN: (.+)/           and $testdsn = $1;
 			/User: (\w+)/         and $testuser = $1;
 			/Helpconnect: (\d+)/  and $helpconnect = $1;
@@ -596,7 +599,7 @@ sub get_test_settings {
 			/Testdir: (.+)/       and $testdir = $1;
 			/pg_ctl: (.+)/        and $pg_ctl = $1;
 			/initdb: (.+)/        and $initdb = $1;
-			/ERROR: (.+)/         and $error = $1;
+			/ERROR: (.+)/         and $error = $1 and $inerror = 1;
 		}
 		close $fh or die qq{Could not close "$helpfile": $!\n};
 	}
